Package: Rrepest
Title: An Analyzer of International Large Scale Assessments in
        Education
Version: 1.6.14
Authors@R: 
    c(person("Rodolfo", "Ilizaliturri", , "rodolfo.ilizaliturri@oecd.org", role = c("aut", "cre")),
    person("Francesco", "Avvisati", , "francesco.avvisati@oecd.org", role = c("aut")),
    person("Francois", "Keslair", , "francois.keslair@oecd.org", role = c("aut")))
Description: An easy way to analyze international large-scale assessments and surveys in education or any other dataset that includes replicated weights (Balanced Repeated Replication (BRR) weights, Jackknife replicate weights,...) while also allowing for analysis with multiply imputed variables (plausible values). It supports the estimation of univariate statistics (e.g. mean, variance, standard deviation, quantiles), frequencies, correlation, linear regression and any other model already implemented in R that takes a data frame and weights as parameters. It also includes  options to prepare the results for publication, following the table formatting standards of the Organization for Economic Cooperation and Development (OECD).
Depends: R (>= 4.2.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: data.table (>= 1.14.8), doParallel (>= 1.0.17), dplyr (>=
        1.1.2), flextable (>= 0.7.2), foreach (>= 1.5.2), labelled (>=
        2.9.1), magrittr (>= 2.0.3), officer (>= 0.6.2), parallel (>=
        4.2.1), purrr (>= 0.3.4), rlang (>= 1.1.6), stringr (>= 1.5.0),
        tibble (>= 3.1.8), tidyr (>= 1.2.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-12 08:52:02 UTC; Ilizaliturri_R
Author: Rodolfo Ilizaliturri [aut, cre],
  Francesco Avvisati [aut],
  Francois Keslair [aut]
Maintainer: Rodolfo Ilizaliturri <rodolfo.ilizaliturri@oecd.org>
Repository: CRAN
Date/Publication: 2026-06-12 13:10:12 UTC
