# Generated by roxygen2: do not edit by hand

export(Rrepest)
export(coverage_daggers)
export(coverage_pct)
export(est)
export(format_data_categ_vars)
export(format_data_cont_vars)
export(format_data_repest)
export(grouped_sum_freqs)
export(grp)
export(indep_diff)
export(inv_test)
export(is_prime)
export(n_obs_x)
export(paired_indep_diff)
export(weighted.corr)
export(weighted.corr.cov.n)
export(weighted.cov)
export(weighted.iqr)
export(weighted.mode)
export(weighted.quant)
export(weighted.std)
export(weighted.var)
import(data.table, except = c(first,last,between,transpose))
import(doParallel)
import(dplyr)
import(flextable)
import(foreach)
import(labelled)
import(magrittr, except = c(set_names,extract))
import(officer)
import(parallel)
import(purrr, except = c(when,compose,accumulate))
import(rlang, except = c("flatten_lgl","splice","flatten_chr","flatten_raw","flatten","flatten_dbl","invoke","flatten_int",":=","%@%"))


import(stringr)
import(tibble)
import(tidyr)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,glm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,na.omit)
importFrom(stats,qnorm)
importFrom(stats,quasibinomial)
importFrom(stats,resid)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
importFrom(utils,combn)
importFrom(utils,head)
