#' Subset of PISA 2018 student questionnaire database
#'
#' This dataset is a subset of the PISA 2018 database produced by the OECD for the countries of France, Italy, and Mexico.
#'
#' @docType data
#' @keywords datasets
#' @name df_pisa18
#' @usage data(df_pisa18)
#' @format A data frame with 1269 rows/observations and 1120 columns/variables.
NULL

#' Subset of TALIS 2018 teacher database
#'
#' This dataset is a subset of the TALIS 2018 database produced by the OECD for the countries of France, Italy, and Mexico.
#'
#' @docType data
#' @keywords datasets
#' @name df_talis18
#' @usage data(df_talis18)
#' @format A data frame with 548 rows/observations and 496 columns/variables.
NULL
