utils::globalVariables(c("idpop", "small.sum'", "big.sum",".",
                         "idschool","school.n","cntschid","schoolid",
                         "schid","small.sum","w.i","data.par","by.group","df.qqq"))
